//**************************************************************************//
//																			//
// File:		Initialisation.c											//
//																			//
// Project:		ADSP-2191 Host Port booting									//
//																			//
// Description: This file contains initialisation functions for Host DSP	//
//				and the Altera PLD.											//
//																			//
//**************************************************************************//

#include "ADSP-2191 Host Port Booting.h"

//--------------------------------------------------------------------------//
// Function:	Init_Host													//
//																			//
// Parameters:	None														//
//																			//
// Return:		None														//
//																			//
// Description: This function initialises External Port, GPIO pins and		//
//				interrupts on the Host DSP.									//
//				The configuration is as follows:							//
//				External Port												//
//				   - data format 16 bit										//
//				   - bus width 16 bit										//
//				   - 7 internal waitstates for MS0 (Flash)					//
//				   - external acknowledge for IOMS (PLD)					//
//				GPIO														//
//				   - PF0 (macro RESET_PF) is output							//
//				   - PF1 (macro ERROR_PF) is input							//
//				   - interrupt on rising edge of ERROR_PF					//
//				Interrupts													//
//				   - FlagA interrupt enabled (for error signal from PLD)	//
//--------------------------------------------------------------------------//
void Init_Host(void)
{
	// configure External Port
	sysreg_write(sysreg_IOPG, External_Access_Bridge_Page);
	io_space_write(E_STAT, 0x0000);				// 16-bit bus; use MSx for access
	sysreg_write(sysreg_IOPG, External_Memory_Interface_Page);
	io_space_write(EMICTL, 0x0038);				// bus width is 16 bit; negative strobe logic
	io_space_write(MS0CTL, 0x007f);				// Flash setting; 7 internal wait states only
	io_space_write(IOMSCTL, 0x0800);			// PLD setting; external acknowledge only

	// configure direction and interrupt sensitivity for flags used as RESET and ERROR
	sysreg_write(sysreg_IOPG, General_Purpose_IO);
	io_space_write(DIR, RESET_PF);				// RESET_PF is output, ERROR_PF is input
	io_space_write(MASKAS, ERROR_PF);			// enable interrupt a for ERROR_PF
	io_space_write(FSSR, ERROR_PF);				// configure ERROR_PF as edge sensitive
	io_space_write(FSBER, 0x0000);				// interrupt for ERROR_PF on one edge
	io_space_write(FSPR, 0x0000);				// interrupt for ERROR_PF on rising edge

	// configure interrupt priorities
	sysreg_write(sysreg_IOPG, Interrupt_Controller_Page);
	io_space_write(IPR0, 0xbbbb);				// set interrupts to lowest priority
	io_space_write(IPR1, 0xbbbb);
	io_space_write(IPR2, 0xbbbb);
	io_space_write(IPR3, 0xbbb4);				// set priority for FlagA interrupt
	interrupt(SIG_INT8, FlagA_ISR);				// assign FlagA ISR to interrupt vector
	enable_interrupts();
}

//--------------------------------------------------------------------------//
// Function:	Init_PLD													//
//																			//
// Parameters:	None														//
//																			//
// Return:		None														//
//																			//
// Description: This function resets the PLD and the Target DSP. This is	//
//				done using the uP Supervisor on the Target EZ-KIT.			//
//				The hardware connection can be found in the application		//
//				note.														//
//				The timing implemented in this function follows the values	//
//				given in the ADM708 data sheet.								//
//--------------------------------------------------------------------------//
void Init_PLD(void)
{
	volatile long l_Delay;

	sysreg_write(sysreg_IOPG, General_Purpose_IO);

	// assert reset for Target EZ-KIT and PLD
	io_space_write(FLAGC, RESET_PF);

	// wait for at least 500 ns (required for uP supervisor)
	for(l_Delay = 0; l_Delay < 0x00ff; l_Delay++);

	// deassert reset
	io_space_write(FLAGS, RESET_PF);

	// wait for at least 280 ms (required for uP supervisor)
	for(l_Delay = 0; l_Delay < 0x000fffff; l_Delay++);

	// read revision number from PLD
	printf("\n\nPLD Revision %d detected.\n", Get_PLD_Version());
}

