---------------------------------------------------------------------------------------------------
--
-- Name:		Rising edge detector module
--
-- Project:		ADSP-2191 Host Port Booting
-- File Name:	rising_edge.vhd
--
---------------------------------------------------------------------------------------------------
--
-- Inputs:		i_clock
--					Clock for rising edge detector
--				i_reset
--					Reset for rising edge detector
--				i_input
--					Input signal
--
-- Outputs:		o_output
--					Output signal
--
-- Description:	This module implements a rising edge detector. The output signal is asserted for
--				one clock cycle when a rising edge is detected on the input signal.
--
---------------------------------------------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE work.pack_const.ALL;

-- entity for rising edge detector
ENTITY rising_edge IS
	PORT(
		i_clock			: IN	STD_LOGIC;	-- edge detector runs at system clock
		i_reset			: IN	STD_LOGIC;	-- reset for edge detector
		i_input			: IN	STD_LOGIC;	-- input signal
		o_output		: OUT	STD_LOGIC	-- output signal
	);
END rising_edge;

-- architecture for rising edge detector
ARCHITECTURE rising_edge OF rising_edge IS
BEGIN
	edge_detection: PROCESS(i_reset, i_clock)
		VARIABLE old_value : STD_LOGIC;
	BEGIN
		-- reset all registered values
		IF (i_reset = ASSERTED) THEN
			old_value	:= i_input;
			o_output	<= DEASSERTED;
		-- check for risging edge of clock
		ELSIF (i_clock'EVENT AND i_clock = ASSERTED) THEN
			-- generate one pulse on detection of rising edge
			IF (old_value = DEASSERTED AND i_input = ASSERTED) THEN
				o_output <= ASSERTED;
			ELSE
				o_output <= DEASSERTED;
			END IF;
			old_value := i_input;
		END IF;
	END PROCESS edge_detection;
END rising_edge;
